@@ -11,6 +11,7 @@ use std::iter::FromIterator;
 use std::iter::IntoIterator;
 use std::mem;
 use std::ops::{Index, IndexMut};
+use alloc::boxed::Box;
 use alloc::vec::Vec;
 
 use crate::imp_prelude::*;
@@ -148,6 +149,18 @@ where
 {
 }
 
+impl<A, S> From<Box<[A]>> for ArrayBase<S, Ix1>
+where
+    S: DataOwned<Elem = A>,
+{
+    /// Create a one-dimensional array from a boxed slice (no copying needed).
+    ///
+    /// **Panics** if the length is greater than `isize::MAX`.
+    fn from(b: Box<[A]>) -> Self {
+        Self::from_vec(b.into_vec())
+    }
+}
+
 impl<A, S> From<Vec<A>> for ArrayBase<S, Ix1>
 where
     S: DataOwned<Elem = A>,
@@ -176,7 +189,6 @@ where
     ///
     /// ```rust
     /// use ndarray::{Array, arr1};
-    /// use std::iter::FromIterator;
     ///
     /// // Either use `from_iter` directly or use `Iterator::collect`.
     /// let array = Array::from_iter((0..5).map(|x| x * x));
@@ -355,6 +367,15 @@ where
     }
 }
 
+impl<A, D> From<Array<A, D>> for ArcArray<A, D>
+where
+    D: Dimension,
+{
+    fn from(arr: Array<A, D>) -> ArcArray<A, D> {
+        arr.into_shared()
+    }
+}
+
 /// Argument conversion into an array view
 ///
 /// The trait is parameterized over `A`, the element type, and `D`, the
