@@ -227,9 +227,32 @@ where
     {
         let n = diag.len();
         let mut arr = Self::zeros((n, n));
-        arr.diag_mut().assign(&diag);
+        arr.diag_mut().assign(diag);
         arr
     }
+
+    /// Create a square 2D matrix of the specified size, with the specified
+    /// element along the diagonal and zeros elsewhere.
+    ///
+    /// **Panics** if `n * n` would overflow `isize`.
+    ///
+    /// ```rust
+    /// use ndarray::{array, Array2};
+    ///
+    /// let array = Array2::from_diag_elem(2, 5.);
+    /// assert_eq!(array, array![[5., 0.], [0., 5.]]);
+    /// ```
+    pub fn from_diag_elem(n: usize, elem: A) -> Self
+    where
+        S: DataMut,
+        A: Clone + Zero,
+    {
+        let mut eye = Self::zeros((n, n));
+        for a_ii in eye.diag_mut() {
+            *a_ii = elem.clone();
+        }
+        eye
+    }
 }
 
 #[cfg(not(debug_assertions))]
@@ -269,7 +292,7 @@ macro_rules! size_of_shape_checked_unwrap {
 /// column major (“f” order) memory layout instead of the default row major.
 /// For example `Array::zeros((5, 6).f())` makes a column major 5 × 6 array.
 ///
-/// Use [`IxDyn`](type.IxDyn.html) for the shape to create an array with dynamic
+/// Use [`type@IxDyn`] for the shape to create an array with dynamic
 /// number of axes.
 ///
 /// Finally, the few constructors that take a completely general
